// LMonomial.h

#ifndef INCLUDED_MONOMIAL_H
#define INCLUDED_MONOMIAL_H

#define NO_DATA

#ifndef NO_DATA
class AuxilaryData;
#endif

#ifndef INCLUDED_VARIABLE_H
#include "Variable.hpp"
#endif
#include "VariableSet.hpp"
#ifndef INCLUDED_DEBUG1_H
#include "Debug1.hpp"
#endif
#include "ICopy.hpp"
#ifndef INCLUDED_GBLIST_H
#include "GBList.hpp"
#endif
#pragma warning(disable:4786)
#include "Choice.hpp"
#ifdef HAS_INCLUDE_NO_DOTS
#include <vector>
#else
#include <vector.h>
#endif
#include "vcpp.hpp"

typedef GBList<Variable>::const_iterator MonomialIterator;

class Monomial {
  friend class OKFriend;
public:
  Monomial();
  Monomial(const Monomial &);
  ~Monomial();
  void  operator = (const Monomial &);

  void reserve(int){};

  void variablesIn(VariableSet &) const;
  void setToOne();
  bool one() const { return _numberOfFactors==0;}
  inline int numberOfFactors() const;
  void setPart(Monomial & m,int start,int end) const;

  // basic math functions
  void operator *= (const Monomial &);
  void operator *= (const Variable &);
  void premultiply(const Monomial & pre);
  void postmultiply(const Monomial & post) {
    operator *=(post);
  };
  void copyFirst(const Monomial &,int);
  void copyLast(const Monomial &,int);
  void prepostmultiply(const Monomial & pre,const Monomial & post);
  bool operator != (const Monomial &) const;
  inline bool operator == (const Monomial &) const;
  MonomialIterator begin() const;
  MonomialIterator end() const;
  int numberMatch(const Monomial &,int start) const;
#ifndef NO_DATA
  AuxilaryData * auxilaryData() const {
    return d_monomialData;
  };
  void auxilaryData(AuxilaryData * p) const;
#endif 
  bool mightBeDivisibleBy(const Monomial & x) const {
    return _numberOfFactors>=x._numberOfFactors;
  };
private:
  inline int compareMonomials(const Monomial &) const;
  void error() const;
#ifndef NO_DATA
  mutable AuxilaryData  * d_monomialData;
#endif
  GBList<Variable> _factors;
  int compareMonomialsAux(const Monomial & aMonomial) const;
  inline void addFactor(const Variable & aTree);
#ifdef CAREFUL
  void check(int index) const;
#endif
  int _numberOfFactors;
};

MyOstream & operator <<(MyOstream &,const Monomial &);

inline void Monomial::operator *= (const Variable & aTree) { 
   if(_numberOfFactors!=_factors.size()) DBG();
  addFactor(aTree);
   if(_numberOfFactors!=_factors.size()) DBG();
};

inline MonomialIterator Monomial::begin() const { 
   if(_numberOfFactors!=_factors.size()) DBG();
  const GBList<Variable> & L = _factors;
  MonomialIterator result = L.begin();
  return result;
};

inline int Monomial::numberOfFactors() const {
   if(_numberOfFactors!=_factors.size()) DBG();
  return _numberOfFactors;
};

inline void Monomial::addFactor(const Variable & aTree) {
   _factors.push_back(aTree);
   _numberOfFactors++;
   if(_numberOfFactors!=_factors.size()) DBG();
// mxs
   if(numberOfFactors()>1000) DBG();
};

inline bool Monomial::operator!= (const Monomial & aMonomial) const { 
   if(_numberOfFactors!=_factors.size()) DBG();
  return !(numberOfFactors()==aMonomial.numberOfFactors() &&
           compareMonomials(aMonomial)==0);
};

inline bool Monomial::operator == (const Monomial & aMonomial) const {
  return numberOfFactors()==aMonomial.numberOfFactors() &&
         compareMonomials(aMonomial)==0;
   if(_numberOfFactors!=_factors.size()) DBG();
};

inline int Monomial::compareMonomials(const Monomial & Right) const {
  int Result;
  if (this == &Right) {
    Result = 0;
  } else {
    Result = _numberOfFactors - Right._numberOfFactors;
    if(Result==0) Result = compareMonomialsAux(Right);
  }
  return Result;
   if(_numberOfFactors!=_factors.size()) DBG();
};
#endif
