// Mark Stankus 1999 (c)
// RegisterRecipients.cpp

#include "CntSource.hpp"
#include "FactControl.hpp"
#include "TripleList.hpp"
#include "FCGiveNumber.hpp"
#include "ICopy.hpp"
#include "Tag.hpp"
#include "RuleID.hpp"
#include "PDReduction.hpp"
#include "PDPolySource.hpp"
#include "GBList.hpp"
#include "Polynomial.hpp"
#include "GroebnerRule.hpp"
#include "ReportReduced.hpp"
#include "PrintGBList.hpp"
#include "Command.hpp"
#include "Polynomial.hpp"
#include "GBList.hpp"
#include "BroadCast.hpp"
#include "Debug1.hpp"
#include "GBLoop.hpp"
#include "Source.hpp"
#include "Sink.hpp"
#include "stringGB.hpp"
#include "Ownership.hpp"
#include "Pools.hpp"
#include "GBHistory.hpp"
#include "RecordHistory.hpp"
#include "Commands.hpp"
#include "RecordToPolySource.hpp"
#include "RecordToReduction.hpp"
#include <set>


void _AddDefaultToPool(Source & so,Sink & si) {
  stringGB pool_name,item_name;
  so >> pool_name >> item_name;
  so.shouldBeEnd();
  si.noOutput();
  AddDefaultToPool(pool_name.value(),item_name.value());
};

AddingCommand temp1RegisterRecipients("AddDefaultToPool",2,_AddDefaultToPool);

void _RemoveFromPool(Source & so,Sink & si) {
  stringGB pool_name,item_name;
  so >> pool_name >> item_name;
  so.shouldBeEnd();
  si.noOutput();
  RemoveFromPool(pool_name.value(),item_name.value());
};

AddingCommand temp1aRegisterRecipients("RemoveFromPool",2,_RemoveFromPool);

void _IncludeReportReduced(Source & so,Sink & si) {
  stringGB broadcast_name;
  so >> broadcast_name;
  so.shouldBeEnd();
  si.noOutput();
  BroadCast * p = Pools<BroadCast>::s_lookup_error(GBStream,broadcast_name.value());
  if(p) {
    ICopy<Recipient> item(new ReportReduced(GBStream),Adopt::s_dummy);
    p->d_L.push_back(item);
  } else {
    GBStream << "Reporting of the reduced polynomials not accepted.\n";
  };
};

AddingCommand temp2RegisterRecipients("IncludeReportReduced",1,
                           _IncludeReportReduced);

void _IncludeRecordPolynomialPolySource(Source & so,Sink & si) {
  stringGB broadcast_name,polysource_name;
  so >> broadcast_name >> polysource_name;
  so.shouldBeEnd();
  si.noOutput();
  BroadCast * bro_p = Pools<BroadCast>::s_lookup_error(GBStream,broadcast_name.value());
  PolySource * ps_p = Pools<PolySource>::s_lookup_error(GBStream,
             polysource_name.value());
  if(!!bro_p && !!ps_p) {
    ICopy<Recipient> item(new RecordToPolySource(*ps_p),Adopt::s_dummy);
    bro_p->d_L.push_back(item);
  } else {
    GBStream << "Reporting of the reduced polynomials not accepted.\n";
  };
};

AddingCommand temp2aRegisterRecipients("IncludeRecordPolynomialPolySource",2,
                           _IncludeRecordPolynomialPolySource);

void _IncludeRecordReduce(Source & so,Sink & si) {
  stringGB broadcast_name,reduction_name;
  so >> broadcast_name >> reduction_name;
  so.shouldBeEnd();
  si.noOutput();
  BroadCast * p = Pools<BroadCast>::s_lookup_error(GBStream,broadcast_name.value());
  Reduction * q = Pools<Reduction>::s_lookup_error(GBStream,
          reduction_name.value());
  if(!!p && !!q) {
    ICopy<Recipient> item(new RecordToReduction(*q),Adopt::s_dummy);
    p->d_L.push_back(item);
  } else {
    GBStream << "Reporting of the reduced polynomials not accepted.\n";
  };
};

AddingCommand temp2bRegisterRecipients("IncludeRecordReduce",1,
                           _IncludeRecordReduce);



void _IncludeRecordGBHistory(Source & so,Sink & si) {
  stringGB broadcast_name, gbhistory_name;
  so >> broadcast_name >> gbhistory_name;
  so.shouldBeEnd();
  si.noOutput();
  list<GBHistory> * p = Pools<list<GBHistory> >::s_lookup_error(GBStream,
         gbhistory_name.value());
  BroadCast * bro = Pools<BroadCast>::s_lookup_error(GBStream,broadcast_name.value());
  if(p) {
    ICopy<Recipient> item(new RecordHistory(*p),Adopt::s_dummy);
    bro->d_L.push_back(item);
  } else {
    GBStream << "Recording of history not accepted.\n";
  };
};

AddingCommand temp3RegisterRecipients("IncludeRecordGBHistory",1,
                           _IncludeRecordGBHistory);

bool InternalCallGBLoop(int num,
     const simpleString & poly_source_name,
     const simpleString &  poly_source_tag_name,
     const simpleString &  reduce_name,
     const simpleString &  reduce_tag_name,
     const simpleString &  broadcast_name,
     const simpleString &  giveNumber_name) {
  PolySource * poly_source_p = Pools<PolySource>::s_lookup_error(GBStream,poly_source_name);
  Tag * polysource_tag_p =  Pools<Tag>::s_lookup_error(GBStream,poly_source_tag_name);
  Reduction * reduce_p = Pools<Reduction>::s_lookup_error(GBStream,reduce_name);
  Tag * reduce_tag_p =  Pools<Tag>::s_lookup_error(GBStream,reduce_tag_name);
  BroadCast * broadcast_p = Pools<BroadCast>::s_lookup_error(GBStream,broadcast_name);
  GiveNumber * give_p = Pools<GiveNumber>::s_lookup_error(GBStream,giveNumber_name);
  bool foundSomething = false;
  if(!!poly_source_p &&!!polysource_tag_p && !!reduce_p && 
     !!reduce_tag_p && !!broadcast_p) {
    for(int i=1;i<=num;++i) {
      GBStream << "Starting iteration " << i << " of " << num << '\n';
      GBLoop(*poly_source_p,*polysource_tag_p,*reduce_p,reduce_tag_p,
           *broadcast_p,foundSomething,*give_p);
      poly_source_p->fillForUnknownReason();
      GBStream << "Ending iteration " << i << " of " << num << '\n';
    };
  } else {
    GBStream << "GBLoop not run due to above mentioned error(s).\n";
  };
  return foundSomething;
};


void _CallGBLoop(Source & so,Sink & si) {
  stringGB poly_source_name,poly_source_tag_name,
           reduce_name,reduce_tag_name,
           broadcast_name,giveNumber_name;
  int num;
  so >> num >> poly_source_name >> poly_source_tag_name >> reduce_name 
     >> reduce_tag_name >> broadcast_name >> giveNumber_name;
  so.shouldBeEnd();
  si << InternalCallGBLoop(num,poly_source_name.value(),
        poly_source_tag_name.value(),
        reduce_name.value(),reduce_tag_name.value(),
        broadcast_name.value(),giveNumber_name.value());
};

AddingCommand temp4RegisterRecipients("CallGBLoop",7,_CallGBLoop);

void _PrintPoolNames(Source & so,Sink & si) {
  stringGB pool_name;
  so >> pool_name;
  so.shouldBeEnd();
  si.noOutput();
  PrintNames(pool_name.value());
};

AddingCommand temp5RegisterRecipients("PrintPoolNames",1,_PrintPoolNames);

void PrintAllPools();

void _PrintAllPools(Source & so,Sink & si) {
  so.shouldBeEnd();
  si.noOutput();
  PrintAllPools();
};

AddingCommand temp6RegisterRecipients("PrintAllPools",0,_PrintPoolNames);

void _AddPolynomialsToGBListPool(Source & so,Sink & si) {
  stringGB name;
  so >> name;
  GBList<Polynomial> * ptr = Pools<GBList<Polynomial> >::s_lookup(name.value());
  Polynomial p;
  if(ptr) {
    Source so2(so.inputNamedFunction("List"));
    while(!so2.eoi()) {
      so2 >> p;
      ptr->push_back(p);     
    };
  } else {
    GBStream << "The name \"" << name << "\" is not recofnized.\n";
    DBG();
  };
  so.shouldBeEnd();
  si.noOutput();
};

AddingCommand temp7RegisterRecipients("AddPolynomialsToGBListPool",2,
             _AddPolynomialsToGBListPool);

void _PrintPolynomialGBList(Source & so,Sink & si) {
  stringGB name;
  so >> name;
  GBList<Polynomial> * ptr = Pools<GBList<Polynomial> >::s_lookup(name.value());
  Polynomial p;
  if(ptr) {
    PrintGBList(name.value().chars(),*ptr);
  } else {
    GBStream << "Cannot print the polynomials with the name \""
             << name << "\"\n";
  };
  so.shouldBeEnd();
  si.noOutput();
};

AddingCommand temp8RegisterRecipients("PrintPolynomialGBList",1,
  _PrintPolynomialGBList);

TripleList<int> * s_TripleList = 0;

void _CreatePolySource(Source & so,Sink & si) {
  if(!s_TripleList) {
    s_TripleList = new TripleList<int>;
  };
  int count;
  stringGB poly_name, source_name, givenumber_name;
  so >> count >> poly_name >> source_name >> givenumber_name;  
  so.shouldBeEnd();
  si.noOutput();
  GBList<Polynomial> * p = 
      Pools<GBList<Polynomial> >::s_lookup_error(GBStream,poly_name.value());
  GiveNumber * give_p = 
      Pools<GiveNumber>::s_lookup_error(GBStream,givenumber_name.value());
  if(!!p && !!give_p) {
    typedef GBList<Polynomial>::const_iterator LI;
    LI w = p->begin(), e = p->end();
    GroebnerRule ru;
    set<RuleID,less<RuleID> > inactive,active;
    int num;
    while(w!=e) {
      const Polynomial & poly = *w;
      num = (*give_p)(poly);
      give_p->setPerm(num);
      s_TripleList->addItem(num);
      RuleID id(poly,num);
      active.insert(id);
      ++w;
    }; 
    ICopy<SetSource<pair<int,int> > > ic((*s_TripleList)());
    SetSource<pair<int,int> > * cnt = new 
       CntSource<pair<int,int>,TripleList<int> >(count,ic,*s_TripleList);
    ICopy<SetSource<pair<int,int> > > icc(cnt,Adopt::s_dummy);
    PDPolySource * ps = new PDPolySource(*give_p,
         active,inactive,icc);
    pair<const simpleString,PolySource *> pr(source_name.value(),ps);
    Pools<PolySource>::s_map.insert(pr);
  } else {
    GBStream << "Cannot execute command due to error. (See above).\n";
  };
};

AddingCommand temp9RegisterRecipients("CreatePolySource",3,
     _CreatePolySource);

void _CreateReduction(Source & so,Sink & si) {
  stringGB reduce_name,giveNumber_name;
  so >> reduce_name >> giveNumber_name;  
  GiveNumber * give_p = 
      Pools<GiveNumber>::s_lookup_error(GBStream,giveNumber_name.value());
  if(!give_p) DBG();
  so.shouldBeEnd();
  si.noOutput();
  PDReduction * p = new PDReduction(*give_p);
  if(!!p) {
    const vector<int> & V = give_p->perm_numbers();
    vector<int>::const_iterator w = V.begin(), e = V.end();
    while(w!=e) {
      SPIID id(*w);
      p->addFact(id);
      ++w;
    };
    pair<const simpleString,Reduction*> pr(reduce_name.value(),p);
    Pools<Reduction>::s_map.insert(pr);
  } else {
    DBG();
  };
};

AddingCommand temp10RegisterRecipients("CreateReduction",2,
     _CreateReduction);

void _CreateReduceTagInfinity(Source & so,Sink & si) {
  stringGB tag_name;
  so >> tag_name;
  so.shouldBeEnd();
  si.noOutput();
  IntTag * p = new IntTag;
  pair<const simpleString,Tag*> pr(tag_name.value(),p);
  Pools<Tag>::s_map.insert(pr);
};

AddingCommand temp11RegisterRecipients("CreateReduceTagInfinity",1,
     _CreateReduceTagInfinity);

void _CreatePolySourceTagInfinity(Source & so,Sink & si) {
  stringGB tag_name;
  so >> tag_name;
  so.shouldBeEnd();
  si.noOutput();
  IntTag * p = new IntTag;
  pair<const simpleString,Tag*> pr(tag_name.value(),p);
  Pools<Tag>::s_map.insert(pr);
};

AddingCommand temp12RegisterRecipients("CreatePolySourceTagInfinity",1,
     _CreatePolySourceTagInfinity);

void _CreateFCNumber(Source & so,Sink & si) {
  stringGB give_name,fc_name;
  so >> give_name >> fc_name;
  so.shouldBeEnd();
  si.noOutput();
  FactControl * fc_p = Pools<FactControl>::s_lookup_error(GBStream,
      fc_name.value());
  FCGiveNumber * fcgive_p = new FCGiveNumber(*fc_p);
  pair<const simpleString,GiveNumber *> pr(give_name.value(),fcgive_p);
  Pools<GiveNumber>::s_map.insert(pr);
};

AddingCommand temp13RegisterRecipients("CreateFCNumber",2,
     _CreateFCNumber);

void _PrintAnswer(Source & so,Sink & si) {
  stringGB fc_name;
  so >> fc_name;
  so.shouldBeEnd();
  si.noOutput();
  FactControl * fc_p = Pools<FactControl>::s_lookup_error(GBStream,
         fc_name.value());
  if(!fc_p) DBG();
  int num = fc_p->numberOfFacts();
  GBStream << "There are  " << num << " facts.\n";
  const GBList<int> & L = fc_p->indicesOfPermanentFacts();
  GBList<int>::const_iterator w = L.begin();
  const int sz = L.size();
  GBStream << "There are  " << sz << " permanent facts.\n";
  for(int i=1;i<=sz;++i,++w) {
    GBStream << fc_p->fact(*w) << '\n';
  };
};

AddingCommand temp14RegisterRecipients("PrintAnswer",2,
     _PrintAnswer);
